#include <FK/FK.h>
#include "TrackBall.h"
#include "OBB.h"
#include "FrameController.h"

int main(int argc, char *argv[])
{
	Fl_Window		mainWindow(800, 600, "COLLISION TEST");
	fk_Model		camera, lightModel, sphereModel;
	fk_Light		light;
	fk_Sphere		sphere(4, 2.0);
	fk_Scene		scene;
	fk_Window		window(0, 0, 800, 600);

	fk_TrackBall	tb(&window, &camera);
	fk_Vector		spd, acc, view, norm, back, refV;
	fk_Matrix		rotMat;

	fk_Block		block[7], playerBox(4.0, 4.0, 4.0);
	fk_Model		blockModel[7];
	fkut_OBB		obb[7], playerOBB;

	fkut_FrameController	fc;

	mainWindow.end();
	fk_InitMaterial();

	// Ɩ̐ݒ
	lightModel.setShape(&light);
	lightModel.setMaterial(White);
	lightModel.glMoveTo(0.0, 0.0, 0.0);
	lightModel.glFocus(0.5, -1.0, 0.0);

	// ̂̐ݒ
	obb[0].setBlockSize(100.0, 100.0, 100.0);
	obb[0].setModel(&blockModel[0]);
	block[0].setSize(100.0, 100.0, 100.0);
	blockModel[0].glMoveTo(0.0, -50.0, 0.0);
	blockModel[0].setMaterial(Yellow);

	obb[1].setBlockSize(20.0, 10.0, 10.0);
	obb[1].setModel(&blockModel[1]);
	block[1].setSize(20.0, 10.0, 10.0);
	blockModel[1].glMoveTo(20.0, 5.0, 10.0);
	blockModel[1].setMaterial(Green);

	obb[2].setBlockSize(10.0, 50.0, 10.0);
	obb[2].setModel(&blockModel[2]);
	block[2].setSize(10.0, 50.0, 10.0);
	blockModel[2].glMoveTo(30.0, 25.0, -30.0);
	blockModel[2].setMaterial(Green);
	blockModel[2].glRotateWithVec(30.0, 25.0, -30.0, fk_X, FK_PI/2.5);

	obb[3].setBlockSize(30.0, 30.0, 10.0);
	obb[3].setModel(&blockModel[3]);
	block[3].setSize(30.0, 30.0, 10.0);
	blockModel[3].glMoveTo(-20.0, 15.0, -10.0);
	blockModel[3].setMaterial(Green);
	blockModel[3].glRotateWithVec(-20.0, 15.0, -10.0, fk_Y, FK_PI/2.5);

	obb[4].setBlockSize(20.0, 10.0, 40.0);
	obb[4].setModel(&blockModel[4]);
	block[4].setSize(20.0, 10.0, 40.0);
	blockModel[4].glMoveTo(-20.0, 5.0, 10.0);
	blockModel[4].setMaterial(Green);
	blockModel[4].glRotateWithVec(-20.0, 5.0, 10.0, fk_Z, FK_PI/2.5);

	obb[5].setBlockSize(10.0, 10.0, 10.0);
	obb[5].setModel(&blockModel[5]);
	block[5].setSize(10.0, 10.0, 10.0);
	blockModel[5].glMoveTo(0.0, 5.0, 0.0);
	blockModel[5].setMaterial(Green);

	obb[6].setBlockSize(10.0, 10.0, 10.0);
	obb[6].setModel(&blockModel[6]);
	block[6].setSize(10.0, 10.0, 10.0);
	blockModel[6].glMoveTo(7.5, 5.0, 0.0);
	blockModel[6].setMaterial(Green);

	// ̂̐ݒ
	sphereModel.setShape(&playerBox);
	sphereModel.setMaterial(Cyan);
	sphereModel.setSmoothMode(true);
	sphereModel.glMoveTo(0.0, 15.0, 0.0);
	playerOBB.setBlockSize(4.0, 4.0, 4.0);
	playerOBB.setModel(&sphereModel);

	// effBXvCXgɓo^
	scene.entryCamera(&camera);
	for(int i = 0; i < 7; i++) {
		blockModel[i].setShape(&block[i]);
		blockModel[i].setSmoothMode(true);
		scene.entryModel(&blockModel[i]);
	}
	scene.entryModel(&lightModel);
	scene.entryModel(&sphereModel);

	// EBhEփfBXvCXgo^
	window.setScene(&scene);

	// _̈ʒuƎpݒ
	camera.glMoveTo(0.0, 100.0, 100.0);
	camera.glFocus(0.0, 0.0, 0.0);
	camera.glUpvec(0.0, 1.0, 0.0);

	rotMat.makeRot(FK_PI/2.0, fk_Y);

	mainWindow.show();
	window.show();

	while(true) {
		static bool prevJump = false;

		fc.TimeRegular();

		if(mainWindow.visible() == 0) {
			if(Fl::wait() == 0) {
				break;
			} else {
				continue;
			}
		}

		tb.update();

		if(fc.GetDrawFlag()) window.drawWindow();
		if(Fl::check() == 0) break;
		if(window.winOpenStatus() == false) continue;

		// J XZ Ƃɐis
		view = camera.getVec();
		view.y = 0.0;
		view.normalize();

		// Wv̏x XZ ̊x悹
		if(window.getKeyStatus(' ', false) & !prevJump) {
			acc.x = spd.x;
			acc.z = spd.z;
			acc.y = 2.0;
		}
		prevJump = window.getKeyStatus(' ', false);

		// n㎞͓x^A󒆂ł͉x
		spd.init();
		if(window.getSpecialKeyStatus(FK_UP, false)) {
			if(abs(acc.y) > 0.0) acc += view*0.1;
			else spd += view*0.5;
		}
		if(window.getSpecialKeyStatus(FK_DOWN, false)) {
			if(abs(acc.y) > 0.0) acc -= view*0.1;
			else spd -= view*0.5;
		}
		if(window.getSpecialKeyStatus(FK_LEFT, false)) {
			if(abs(acc.y) > 0.0) acc += rotMat*view*0.1;
			else spd += rotMat*view*0.5;
		}
		if(window.getSpecialKeyStatus(FK_RIGHT, false)) {
			if(abs(acc.y) > 0.0) acc -= rotMat*view*0.1;
			else spd -= rotMat*view*0.5;
		}
		if(spd.normalize()) spd *= 0.5;

		// d͉x
		acc.y -= 0.098;
		if(acc.x*acc.x + acc.z*acc.z > 0.5*0.5) {
			double tmpSpdLen = sqrt(acc.x*acc.x + acc.z*acc.z);
			acc.x *= 0.5/tmpSpdLen;
			acc.z *= 0.5/tmpSpdLen;
		} else {
			acc.x *= 0.9;
			acc.z *= 0.9;
		}

		// x𑬓xɏ悹
		spd += acc;

		// Փ˔̊Jn
		for(int i = 0; i < 7; i++) {
			if(i == 0) {
				blockModel[i].setMaterial(Yellow);
			} else {
				blockModel[i].setMaterial(Green);
			}
		}
		do {
			back.init();
			for(int i = 0; i < 7; i++) {
				if(obb[i].checkVSMovingOBB(playerOBB, spd+back, refV)) {
					back += -refV;
					blockModel[i].setMaterial(Red);
				}
			}
			if(abs(back.y) > 0.0) {
				acc.init();
			}
			spd += back;
		} while(!back.isZero());
		// Փ˂̔͂𑬓xɉďI

		// x𑀍샂fɉĎۂɓ
		sphereModel.glTranslate(spd);
		camera.glTranslate(spd.x, 0.0, spd.z);
		tb.setLookTo(fk_Vector(sphereModel.getPosition().x, 0.0, sphereModel.getPosition().z));
	}

	return 0;
}
